#!/usr/bin/env node

import { execSync } from 'child_process';
import { readFileSync, existsSync, access } from 'fs';
import firebase from 'firebase/compat/app';
import 'firebase/compat/firestore';
import 'firebase/compat/auth';
import { intro, outro, spinner, text } from '@clack/prompts';
import capitalizedName from './utils/getUserInfo.js';
import colorText from './utils/textUtils.js';
import axios from 'axios'

const s = spinner();

const firebaseConfig = {
    apiKey: "AIzaSyABm9M7XceFH6pSvvbMuRJw3n5nBeak3L0",
    authDomain: "share-brewfiles.firebaseapp.com",
    projectId: "share-brewfiles",
    storageBucket: "share-brewfiles.appspot.com",
    messagingSenderId: "1071283599196",
    appId: "1:1071283599196:web:a4910843f1bae76d7af399",
    measurementId: "G-WGD9RH2E7M"
  };

firebase.initializeApp(firebaseConfig);

function isHomebrewInstalled() {
    try {
        execSync('brew --version');
        return true;
    } catch (error) {
        return false;
    }
}

async function authenticateWithGitHub() {
    try {
        await startClackSpinner('Before uploading your Brewfile, we ask that you connect it with a Github account.')
        await stopClackSpinner('Please login through the browser window that\'s about to open...')
        execSync('open https://github.com/login/oauth/authorize?client_id=6db8c19033a30994dffa&scope=read:user');
        
        const code = await text({
            message: 'Paste the code you were given here:',
            placeholder: '',
            initialValue: '',
          });

        return code;
    } catch (error) {
        return null;
    }
}

async function exchangeCodeForAccessToken(code) {
    await startClackSpinner('Verifying your code...')
    try {
        const response = await axios.post(`https://github.com/login/oauth/access_token`, {
            client_id: '6db8c19033a30994dffa',
            client_secret: 'b3a184d67b49bdfcd5e33291ab14f378fd5fa52a',
            code: code
        }, {
            headers: {
                'Accept': 'application/json'
            }
        });
        
        const accessToken = response.data.access_token;
        await stopClackSpinner('Verified!')
        return accessToken;
    } catch (error) {
        return null;
    }
}

async function startClackSpinner(text) {
    s.start(text)
    await new Promise(resolve => setTimeout(resolve, 2000));
}

async function stopClackSpinner(text) {
    s.stop(text)
    await new Promise(resolve => setTimeout(resolve, 2000));
}

function installHomebrew() {
    execSync('/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"');
}

async function createNewBrewfile() {
    execSync('brew bundle dump');
}

async function getBrewfile() {
    await startClackSpinner('Checking for existing Brewfile')
    if (existsSync('Brewfile')) {
        await stopClackSpinner("Found existing Brewfile!")
    } else {
        await stopClackSpinner("No existing Brewfile found!")
        await startClackSpinner("Generating Brewfile for you")
        createNewBrewfile()
        await stopClackSpinner("Finished generating new Brewfile!")
    }
}

async function readBrewfileData() {
    const brewfileData = readFileSync('Brewfile', 'utf-8');
    return brewfileData;
}

function getCurrentDate() {
    const currentDate = new Date();
    const day = currentDate.getDate();
    const month = currentDate.getMonth() + 1;
    const year = currentDate.getFullYear();
    
    return `${month}/${day}/${year}`;
}

async function uploadBrewfileToFirebase(brewfileData, accessToken) {
    await startClackSpinner("Uploading Brewfile to our website")
    const db = firebase.firestore();
    const collectionRef = db.collection('brewfiles');
    await collectionRef.add({ data: brewfileData, date: getCurrentDate(), accessToken: accessToken});
    await stopClackSpinner("Successfully uploaded your Brewfile!")
}

async function parseBrewfile(brewfileData) {
    const packages = {};
    const lines = brewfileData.split('\n');
    
    lines.forEach((line, index) => {
        if (line.startsWith('tap')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'tap' };
        } else if (line.startsWith('brew')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'brew' };
        } else if (line.startsWith('vscode')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'vscode' };
        }
    });
    return packages;
}

async function main() {
    try {
        console.clear();
        console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
        console.log()
        intro(`Welcome to ${colorText('Share Brewfiles', "BLACK", "GREEN")}, ${capitalizedName}!`)
        if (!isHomebrewInstalled()) {
            installHomebrew();
        } else {
            await getBrewfile();
            const brewfileData = await readBrewfileData();
            if (brewfileData) {
                const packages = await parseBrewfile(brewfileData);
                const code = await authenticateWithGitHub();
                const accessToken = await exchangeCodeForAccessToken(code)
                if (accessToken) {
                    await uploadBrewfileToFirebase(packages, accessToken);
                }
            }
        }
        outro(`${colorText("Done! Now check out the community leaderboard here:", "BLACK", "GREEN")}`)
        console.log(`👉 https://share-brewfiles-website.vercel.app/ 👈`)
        console.log()
        console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
        process.exit(0);
    } catch (error) {
        console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
        console.log()
        console.log(`Something went wrong! ${colorText("Exiting the program.", "WHITE", "MAGENTA")}`);
        console.log()
        console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
        process.exit(0);
    }
}

main();
